/*
    Exotic Pan Filter for VirtualDub.
	by Natan Never 2000 (NNever2000)
    
	I'm searched this filter for months on all internet sites. All pan filter
	that I found was near but not perfect for what I want.

	So I started coding it myself.

	Because I'm a C++ novice, this filter is based on the project files of
	Donald A. Graft's Zoom Filter (that was the first filter that I 
	compiled successfull). So, all credits goes to Donald Graft and Avery Lee 
	that are the authors of the original, astonishing, Zoom filter.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    The author can be contacted at:

	Natan Never 2000
	nnever2000@yahoo.it

	2014.06.04	Bugfix:Save/Load now called properly so that works
				Mod:Max size limit raised, enabling e.g 1280x720 resolution
				Mod:Adding buttons for standard resolutions.
				Mod:Debug frame drawn with triple pixel frame, assuring visibility also down to 33%
*/

#define VDEXT_VIDEO_FILTER
#define VDEXT_MAIN

#include <windows.h>
#include <commctrl.h>
#include <crtdbg.h>
#include <assert.h>
#include <stdio.h>
#include <math.h>

#include "ScriptInterpreter.h"
#include "ScriptValue.h"
#include "ScriptError.h"

#include "resource.h"
#include "filter.h"

///////////////////////

typedef struct MyFilterData {
	long new_x, new_y;

	int iOffset_x[10];
	int iOffset_y[10];
	int iFrame[10];
	bool fEnabled[10];
	int iIntervals;
	
	// other
	int red, green, blue;
	bool fDebug;
	bool fDontCrop;

	IFilterPreview *ifp;

} MyFilterData;

////////////////////

static int resize_run(const FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	// Get source frame number
	int frame = fa->pfsi->lCurrentFrame;

	int start, end, startfocusx, endfocusx, startfocusy, endfocusy;
	int focusx, focusy;

	// find the interval
	if (frame <= mfd->iFrame[0])
	{
		focusx = mfd->iOffset_x[0];
		focusy = mfd->iOffset_y[0];
	}
	else if (frame >= mfd->iFrame[mfd->iIntervals-1])
	{
		focusx = mfd->iOffset_x[mfd->iIntervals-1];
		focusy = mfd->iOffset_y[mfd->iIntervals-1];
	}
	else
	{
		focusx = 0;
		focusy = 0;

		for (int i = 0; i < mfd->iIntervals-1; i++)
		{
			if (frame>=mfd->iFrame[i] && frame<=mfd->iFrame[i+1]) 
			{
				start = mfd->iFrame[i];
				end = mfd->iFrame[i+1];
				startfocusx = mfd->iOffset_x[i];
				endfocusx = mfd->iOffset_x[i+1];
				startfocusy =  mfd->iOffset_y[i];
				endfocusy =  mfd->iOffset_y[i+1];
				
				focusx = startfocusx + ((endfocusx-startfocusx) * (frame-start)) / (end-start);
				focusy = startfocusy + ((endfocusy-startfocusy) * (frame-start)) / (end-start);
			}
		}
	}

	//double percent;
	Pixel *dst, *src;
	unsigned long color;	

	/* Paint the destination frame with the color fill. */
	dst = fa->dst.data;
	color = (mfd->red << 16) | (mfd->green << 8) | mfd->blue;
	for (int y = 0; y < fa->dst.h; y++)
	{
		for (int x = 0; x < fa->dst.w; x++)
			dst[x] = color;
		dst	= (Pixel *)((char *)dst + fa->dst.pitch);
	}

	// Begin rendering source -> filter -> destination

	// Calcolate the box
	int x_position = focusx;
	int y_position = fa->src.h-focusy-1;
	int w_position = focusx + mfd->new_x - 1;
	int h_position = fa->src.h-(focusy + mfd->new_y);

	// 
	if (mfd->fDontCrop && (frame < mfd->iFrame[0] || frame > mfd->iFrame[mfd->iIntervals-1]))
	{
		dst = fa->dst.data;
		src = fa->src.data;
		for (int y = 0; y < fa->dst.h; y++)
		{
			for (int x = 0; x < fa->dst.w; x++) dst[x] = src[x];
			dst	= (Pixel *)((char *)dst + fa->dst.pitch);
			src	= (Pixel *)((char *)src + fa->src.pitch);
		}
	}
	else
	{
		if (!mfd->fDebug) 
		{
			dst = fa->dst.data;
			src = fa->src.data;

			color = (mfd->red << 16) | (mfd->green << 8) | mfd->blue;	

			// Move to the first source line to render
			src	= (Pixel *)((char *)src + fa->src.pitch * (fa->src.h-focusy-fa->dst.h));

			for (int y = 0; y < fa->dst.h; y++)
			{
				for (int x = 0; x < fa->dst.w; x++)
				{
					if ((x+x_position>=0) && (x+x_position<fa->src.w) &&
						(y>=focusy+fa->dst.h-fa->src.h))
					{
						dst[x] = src[x+x_position];
					}
				}			
				src	= (Pixel *)((char *)src + fa->src.pitch);
				dst	= (Pixel *)((char *)dst + fa->dst.pitch);
			}
		}
		else
		{
			// DEBUG MODE, visualizza il box in colore di fondo
			// Paint the destination frame with the color fill.
			// In debug mode, dst.w = src.w and dst.h=src.h
			dst = fa->dst.data;
			src = fa->src.data;
			color = (mfd->red << 16) | (mfd->green << 8) | mfd->blue;
			//color = (255 << 16) | (0 << 8) | 0; // red color
			int wFrame = 1;
			int y_p3 = y_position/wFrame + 1;
			int h_p3 = h_position/wFrame + 1;
			int x_p3 = x_position/wFrame - 1;
			int w_p3 = w_position/wFrame + 1;
			for (int y = 0; y < fa->dst.h; y++)
			{
				int y3p = (y-wFrame)/wFrame;
				int y3h = y/wFrame;
				for (int x = 0; x < fa->dst.w; x++)
				{
					if ((x>=x_position && x<=w_position && (y3p==y_p3 || y3h==h_p3))     //Horizontal borders
						|| (y<=y_position+3 && y>=h_position+wFrame && ((x-1)/wFrame==x_p3 || (x)/wFrame==w_p3))  //Vertical borders
						//|| ((y%10==0) ||(x%10==0))  //Grid
						)
					{
						dst[x] = color;
					}
					else
					{
						dst[x] = src[x];
					}
				}
				dst	= (Pixel *)((char *)dst + fa->dst.pitch);
				src	= (Pixel *)((char *)src + fa->src.pitch);
			}

		}
	}
	return 0;
}

static long resize_param(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	const int INTERVALS = 10;

	if (!mfd) return 0;

	// verify & correct the filter variables

	// fix the new x and y 
	if (mfd->new_x>fa->src.w) mfd->new_x=fa->src.w;	
	if (mfd->new_y>fa->src.h) mfd->new_y=fa->src.h;
	if (mfd->new_x<16) mfd->new_x=16;	
	if (mfd->new_y<16) mfd->new_y=16;

	// Sort intervals
	for (int x = 0; x < INTERVALS -1; x++)
	{
		for (int y = x + 1; y < INTERVALS; y++)
		{
			if (
				((mfd->fEnabled[x]==mfd->fEnabled[y]) && (mfd->iFrame[x]>mfd->iFrame[y]))
				||
				(mfd->fEnabled[x]==false &&  mfd->fEnabled[y]==true)
				)
			{
				int iFrame_swap, iOffset_x_swap, iOffset_y_swap;
				bool fEnabled_swap;

				iFrame_swap=mfd->iFrame[x];
				mfd->iFrame[x]=mfd->iFrame[y];
				mfd->iFrame[y]=iFrame_swap;

				iOffset_x_swap=mfd->iOffset_x[x];
				mfd->iOffset_x[x]=mfd->iOffset_x[y];
				mfd->iOffset_x[y]=iOffset_x_swap;

				iOffset_y_swap=mfd->iOffset_y[x];
				mfd->iOffset_y[x]=mfd->iOffset_y[y];
				mfd->iOffset_y[y]=iOffset_y_swap;
					
				fEnabled_swap=mfd->fEnabled[x];
				mfd->fEnabled[x]=mfd->fEnabled[y];
				mfd->fEnabled[y]=fEnabled_swap;			
			}				
		}			
	}

	// Count intervals
	mfd->iIntervals=0;
	for (int i = 0; i < INTERVALS; i++)
	{
		if (mfd->fEnabled[i]) mfd->iIntervals++;
	}

	// There will be at least one interval
	if (mfd->iIntervals==0)
	{
		mfd->fEnabled[0]=true;
		mfd->iIntervals=1;
	}

	// fix the enabled offsets
	for (int i = 0; i < mfd->iIntervals; i++)
	{
		//if (mfd->fEnabled[i])
		//{
			if ((mfd->iOffset_x[i] + mfd->new_x) > fa->dst.w) 
				mfd->iOffset_x[i] = fa->dst.w - mfd->new_x;

			if ((mfd->iOffset_y[i] + mfd->new_y) > fa->dst.h) 
				mfd->iOffset_y[i] = fa->dst.h - mfd->new_y;
		//}
	}

	// Screen size is the same of the original in debug mode or dontcrop mode
	if (!mfd->fDebug && !mfd->fDontCrop)
	{
		fa->dst.w = mfd->new_x;
		fa->dst.h = mfd->new_y;
	}

	fa->dst.AlignTo8();

	return FILTERPARAM_SWAP_BUFFERS;
}

static BOOL APIENTRY resizeDlgProc( HWND hDlg, UINT message, UINT wParam, LONG lParam) {
	MyFilterData *mfd = (struct MyFilterData *)GetWindowLong(hDlg, DWL_USER);

    switch (message)
    {
        case WM_INITDIALOG:
			{
				mfd = (MyFilterData *)lParam;
				SetDlgItemInt(hDlg, IDC_WIDTH, mfd->new_x, FALSE);
				SetDlgItemInt(hDlg, IDC_HEIGHT, mfd->new_y, FALSE);

				//0
				SetDlgItemInt(hDlg, IDC_OFFSET_X_0, mfd->iOffset_x[0], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_0, mfd->iOffset_y[0], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_0, mfd->iFrame[0], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_0, mfd->fEnabled[0] ? BST_CHECKED : BST_UNCHECKED);

				//1
				SetDlgItemInt(hDlg, IDC_OFFSET_X_1, mfd->iOffset_x[1], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_1, mfd->iOffset_y[1], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_1, mfd->iFrame[1], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_1, mfd->fEnabled[1] ? BST_CHECKED : BST_UNCHECKED);

				//2
				SetDlgItemInt(hDlg, IDC_OFFSET_X_2, mfd->iOffset_x[2], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_2, mfd->iOffset_y[2], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_2, mfd->iFrame[2], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_2, mfd->fEnabled[2] ? BST_CHECKED : BST_UNCHECKED);
				
				//3
				SetDlgItemInt(hDlg, IDC_OFFSET_X_3, mfd->iOffset_x[3], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_3, mfd->iOffset_y[3], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_3, mfd->iFrame[3], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_3, mfd->fEnabled[3] ? BST_CHECKED : BST_UNCHECKED);

				//4
				SetDlgItemInt(hDlg, IDC_OFFSET_X_4, mfd->iOffset_x[4], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_4, mfd->iOffset_y[4], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_4, mfd->iFrame[4], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_4, mfd->fEnabled[4] ? BST_CHECKED : BST_UNCHECKED);

				//5
				SetDlgItemInt(hDlg, IDC_OFFSET_X_5, mfd->iOffset_x[5], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_5, mfd->iOffset_y[5], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_5, mfd->iFrame[5], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_5, mfd->fEnabled[5] ? BST_CHECKED : BST_UNCHECKED);

				//6
				SetDlgItemInt(hDlg, IDC_OFFSET_X_6, mfd->iOffset_x[6], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_6, mfd->iOffset_y[6], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_6, mfd->iFrame[6], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_6, mfd->fEnabled[6] ? BST_CHECKED : BST_UNCHECKED);

				//7
				SetDlgItemInt(hDlg, IDC_OFFSET_X_7, mfd->iOffset_x[7], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_7, mfd->iOffset_y[7], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_7, mfd->iFrame[7], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_7, mfd->fEnabled[7] ? BST_CHECKED : BST_UNCHECKED);

				//8
				SetDlgItemInt(hDlg, IDC_OFFSET_X_8, mfd->iOffset_x[8], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_8, mfd->iOffset_y[8], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_8, mfd->iFrame[8], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_8, mfd->fEnabled[8] ? BST_CHECKED : BST_UNCHECKED);

				//9
				SetDlgItemInt(hDlg, IDC_OFFSET_X_9, mfd->iOffset_x[9], FALSE);
				SetDlgItemInt(hDlg, IDC_OFFSET_Y_9, mfd->iOffset_y[9], FALSE);	
				SetDlgItemInt(hDlg, IDC_FRAME_9, mfd->iFrame[9], FALSE);
				CheckDlgButton(hDlg, IDC_ENABLED_9, mfd->fEnabled[9] ? BST_CHECKED : BST_UNCHECKED);

				SetDlgItemInt(hDlg, IDC_FILLRED, mfd->red, FALSE);
				SetDlgItemInt(hDlg, IDC_FILLGREEN, mfd->green, FALSE);
				SetDlgItemInt(hDlg, IDC_FILLBLUE, mfd->blue, FALSE);

				CheckDlgButton(hDlg, IDC_DEBUG, mfd->fDebug ? BST_CHECKED : BST_UNCHECKED);

				CheckDlgButton(hDlg, IDC_DONT_CROP, mfd->fDontCrop ? BST_CHECKED : BST_UNCHECKED);				

				SetWindowLong(hDlg, DWL_USER, (LONG)mfd);

				mfd->ifp->InitButton(GetDlgItem(hDlg, IDC_PREVIEW));
			}
            return (TRUE);

        case WM_COMMAND:                      
			switch(LOWORD(wParam)) {
			case IDOK:
				mfd->ifp->Close();
				EndDialog(hDlg, 0);
				return TRUE;

			case IDCANCEL:
				mfd->ifp->Close();
                EndDialog(hDlg, 1);
                return TRUE;

			case IDC_WIDTH:
				if (HIWORD(wParam) == EN_UPDATE) {
					long new_x;
					BOOL success;

					new_x = GetDlgItemInt(hDlg, IDC_WIDTH, &success, FALSE);
					if (!success || new_x < 1 || new_x > 3840) {  //4K is 1920 * 2
						SetDlgItemInt(hDlg, IDC_WIDTH, mfd->new_x, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->new_x = new_x;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_HEIGHT:
				if (HIWORD(wParam) == EN_UPDATE) {
					long new_y;
					BOOL success;

					new_y = GetDlgItemInt(hDlg, IDC_HEIGHT, &success, FALSE);
					if (!success || new_y < 1 || new_y > 2160) {  //4k is 1080 * 2 i think
						SetDlgItemInt(hDlg, IDC_HEIGHT, mfd->new_y, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->new_y = new_y;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			// ************************* 0 *******************************
			case IDC_OFFSET_X_0:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_0, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_0, mfd->iOffset_x[0], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[0] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_0:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_0, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_0, mfd->iOffset_y[0], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[0] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_0:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_0, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_0, mfd->iFrame[0], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[0] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_0:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_0);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[0] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 1 *******************************
			case IDC_OFFSET_X_1:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_1, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_1, mfd->iOffset_x[1], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[1] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_1:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_1, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_1, mfd->iOffset_y[1], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[1] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_1:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_1, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_1, mfd->iFrame[1], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[1] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_1:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_1);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[1] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 2 *******************************
			case IDC_OFFSET_X_2:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_2, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_2, mfd->iOffset_x[2], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[2] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_2:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_2, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_2, mfd->iOffset_y[2], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[2] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_2:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_2, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_2, mfd->iFrame[2], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[2] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_2:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_2);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[2] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 3 *******************************
			case IDC_OFFSET_X_3:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_3, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_3, mfd->iOffset_x[3], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[3] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_3:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_3, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_3, mfd->iOffset_y[3], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[3] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_3:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_3, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_3, mfd->iFrame[3], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[3] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_3:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_3);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[3] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 4 *******************************
			case IDC_OFFSET_X_4:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_4, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_4, mfd->iOffset_x[4], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[4] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_4:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_4, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_4, mfd->iOffset_y[4], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[4] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_4:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_4, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_4, mfd->iFrame[4], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[4] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_4:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_4);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[4] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 5 *******************************
			case IDC_OFFSET_X_5:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_5, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_5, mfd->iOffset_x[5], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[5] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_5:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_5, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_5, mfd->iOffset_y[5], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[5] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_5:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_5, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_5, mfd->iFrame[5], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[5] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_5:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_5);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[5] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 6 *******************************
			case IDC_OFFSET_X_6:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_6, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_6, mfd->iOffset_x[6], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[6] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_6:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_6, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_6, mfd->iOffset_y[6], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[6] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_6:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_6, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_6, mfd->iFrame[6], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[6] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_6:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_6);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[6] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 7 *******************************
			case IDC_OFFSET_X_7:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_7, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_7, mfd->iOffset_x[7], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[7] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_7:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_7, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_7, mfd->iOffset_y[7], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[7] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_7:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_7, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_7, mfd->iFrame[7], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[7] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_7:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_7);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[7] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 8 *******************************
			case IDC_OFFSET_X_8:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_8, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_8, mfd->iOffset_x[8], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[8] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_8:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_8, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_8, mfd->iOffset_y[8], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[8] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_8:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_8, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_8, mfd->iFrame[8], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[8] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_8:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_8);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[8] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************************* 9 *******************************
			case IDC_OFFSET_X_9:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_OFFSET_X_9, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_X_9, mfd->iOffset_x[9], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_x[9] = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_OFFSET_Y_9:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusy;
					BOOL success;

					focusy = GetDlgItemInt(hDlg, IDC_OFFSET_Y_9, &success, FALSE);
					if (!success || focusy > 1024) {
						SetDlgItemInt(hDlg, IDC_OFFSET_Y_9, mfd->iOffset_y[9], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iOffset_y[9] = focusy;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAME_9:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_FRAME_9, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_FRAME_9, mfd->iFrame[9], FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->iFrame[9] = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_ENABLED_9:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_ENABLED_9);

					mfd->ifp->UndoSystem();
					mfd->fEnabled[9] = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			// ************
			case IDC_FILLRED:
				if (HIWORD(wParam) == EN_UPDATE) {
					long red;
					BOOL success;

					red = GetDlgItemInt(hDlg, IDC_FILLRED, &success, FALSE);
					if (!success || red > 255) {
						SetDlgItemInt(hDlg, IDC_FILLRED, mfd->red, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->red = red;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FILLGREEN:
				if (HIWORD(wParam) == EN_UPDATE) {
					long green;
					BOOL success;

					green = GetDlgItemInt(hDlg, IDC_FILLGREEN, &success, FALSE);
					if (!success || green > 255) {
						SetDlgItemInt(hDlg, IDC_FILLGREEN, mfd->green, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->green = green;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FILLBLUE:
				if (HIWORD(wParam) == EN_UPDATE) {
					long blue;
					BOOL success;

					blue = GetDlgItemInt(hDlg, IDC_FILLBLUE, &success, FALSE);
					if (!success || blue > 255) {
						SetDlgItemInt(hDlg, IDC_FILLBLUE, mfd->blue, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->blue = blue;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_PREVIEW:
				mfd->ifp->Toggle(hDlg);
				return TRUE;

			case IDC_DEBUG:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_DEBUG);

					mfd->ifp->UndoSystem();
					mfd->fDebug = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_DONT_CROP:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_DONT_CROP);

					mfd->ifp->UndoSystem();
					mfd->fDontCrop = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

 			case IDHELP:
				{
				char prog[256];
				char path[256];
				LPTSTR ptr;

				GetModuleFileName(NULL, prog, 255);
				GetFullPathName(prog, 255, path, &ptr);
				*ptr = 0;
				strcat(path, "plugins\\ExoticPan.html");
				ShellExecute(hDlg, "open", path, NULL, NULL, SW_SHOWNORMAL);
				return TRUE;
				}

			case IDC_720p:
				{
				SetDlgItemInt(hDlg, IDC_WIDTH, 1280, FALSE);  //1280 x 720
				SetDlgItemInt(hDlg, IDC_HEIGHT, 720, FALSE);  
				return TRUE;
				}

			case IDC_576p:
				{
				SetDlgItemInt(hDlg, IDC_WIDTH, 1024, FALSE);  //720 x 576
				SetDlgItemInt(hDlg, IDC_HEIGHT, 576, FALSE);  
				return TRUE;
				}

			case IDC_480p:
				{
					long new_x;
					BOOL success; //854 / 640 x 480

					new_x = GetDlgItemInt(hDlg, IDC_WIDTH, &success, FALSE);
					if (!success || new_x != 854 || new_x > 3840) {
						SetDlgItemInt(hDlg, IDC_WIDTH, 854, FALSE);  
					} else {
						SetDlgItemInt(hDlg, IDC_WIDTH, 640, FALSE);  
					}
				SetDlgItemInt(hDlg, IDC_HEIGHT, 480, FALSE);  
				return TRUE;
				}

			case IDC_360p:
				{
					long new_x;
					BOOL success; //640 / 480 x 360

					new_x = GetDlgItemInt(hDlg, IDC_WIDTH, &success, FALSE);
					if (!success || new_x != 640 || new_x > 3840) {  
						SetDlgItemInt(hDlg, IDC_WIDTH, 640, FALSE);  
					} else {
						SetDlgItemInt(hDlg, IDC_WIDTH, 480, FALSE);  
					}
				SetDlgItemInt(hDlg, IDC_HEIGHT, 360, FALSE);  
				return TRUE;
				}

			}
            break;
    }
    return FALSE;
}

static int resize_config(FilterActivation *fa, const FilterFunctions *ff, HWND hWnd) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	MyFilterData mfd2 = *mfd;
	int ret;

	mfd->ifp = fa->ifp;

	if (mfd->new_x < 16)
		mfd->new_x = 320;
	if (mfd->new_y < 16)
		mfd->new_y = 240;

	ret = DialogBoxParam(fa->filter->module->hInstModule, MAKEINTRESOURCE(IDD_FILTER_ZOOM),
						 hWnd, resizeDlgProc, (LONG)mfd);

	if (ret)
		*mfd = mfd2;

	return ret;
}

static void resize_string(const FilterActivation *fa, const FilterFunctions *ff, char *buf) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	if (mfd->fDontCrop)
	{
		char framenumber[65]="";
		char framerange[256]="";
		char bufstring[256]="";
		
		_itoa( mfd->iFrame[0], framenumber, 10 ); 
		strcat(framerange, framenumber);
		
		strcat(framerange, "-");
		
		_itoa( mfd->iFrame[mfd->iIntervals-1], framenumber, 10 ); 
		strcat(framerange, framenumber);

		if (mfd->fDebug)
		{
			strcat(bufstring, "Debug mode, no crop ");
			strcat(bufstring, framerange);
		}
		else
		{
			strcat(bufstring, "Normal mode, no crop ");
			strcat(bufstring, framerange);			
		}
		
		wsprintf(buf, " (%s)", bufstring);
		
	}
	else
	{
		if (mfd->fDebug)
		{
			wsprintf(buf, " (%s)", "Debug mode");
		}
		else
		{
			wsprintf(buf, " (%s)", "Normal mode");
		}
	}

}

static int resize_start(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
/*	double dstw = (mfd->startpercent * fa->src.w) / 100.0;
	double dsth = (mfd->startpercent * fa->src.h) / 100.0;

	if (mfd->new_x > fa->src.w || mfd->new_y > fa->src.h)
	{
		MessageBox(NULL,
			"Dimensions of the output frame cannot exceed those of "
			"the input frame. Please correct this and try again."
			,
			"Invalid output window size parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (mfd->end < mfd->start)
	{
		MessageBox(NULL,
			"The specified ending frame cannot be before the specified "
			"starting frame. Please correct this and try again."
			,
			"Invalid ending frame parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (mfd->startpercent2 != 0 && mfd->start2 < mfd->end)
	{
		MessageBox(NULL,
			"The specified phase 2 starting frame cannot be before the "
			"specified phase 1 ending frame. Please correct this and try again."
			,
			"Invalid starting frame parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (mfd->startpercent2 != 0 && mfd->end2 < mfd->start2)
	{
		MessageBox(NULL,
			"The specified phase 2 ending frame cannot be before the "
			"specified phase 2 starting frame. Please correct this and try again."
			,
			"Invalid ending frame parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (mfd->startfocusx > fa->src.w - 1 || mfd->startfocusy > fa->src.h - 1)
	{
		MessageBox(NULL,
			"The focus point must be within the dimensions of "
			"the input frame. Please correct this and try again."
			,
			"Invalid focus parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (dstw<16 || dsth<16)
	{
		MessageBox(NULL,
			"The resampler requires the scaled output area to be at least "
			"16 x 16 pixels. Please correct this and try again."
			,
			"Invalid resampler configuration",
			MB_ICONEXCLAMATION);
		return 1;
	}

	switch(mfd->filter_mode) {
	case FILTER_NONE:			mfd->fmode = Resampler::eFilter::kPoint; break;
	case FILTER_BILINEAR:		mfd->fmode = Resampler::eFilter::kLinearInterp; break;
	case FILTER_BICUBIC:		mfd->fmode = Resampler::eFilter::kCubicInterp; break;
	case FILTER_TABLEBILINEAR:	mfd->fmode = Resampler::eFilter::kLinearDecimate; break;
	case FILTER_TABLEBICUBIC:	mfd->fmode = Resampler::eFilter::kCubicDecimate; break;
	}

	mfd->resampler = new Resampler();

	mfd->resampler->Init(mfd->fmode, mfd->fmode, dstw, dsth, fa->src.w, fa->src.h);*/

	return 0;
}

static int resize_stop(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

//	delete mfd->resampler;	mfd->resampler = NULL;

	return 0;
}

static void resize_script_config(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc)
{
	FilterActivation *fa = (FilterActivation *)lpVoid;

	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->new_x			= argv[0].asInt();
	mfd->new_y			= argv[1].asInt();
	
	mfd->iOffset_x[0]	= argv[2].asInt();
	mfd->iOffset_y[0]	= argv[3].asInt();
	mfd->iFrame[0]		= argv[4].asInt();
	mfd->fEnabled[0]	= argv[5].asInt();

	mfd->iOffset_x[1]	= argv[6].asInt();
	mfd->iOffset_y[1]	= argv[7].asInt();
	mfd->iFrame[1]		= argv[8].asInt();
	mfd->fEnabled[1]	= argv[9].asInt();

	mfd->iOffset_x[2]	= argv[10].asInt();
	mfd->iOffset_y[2]	= argv[11].asInt();
	mfd->iFrame[2]		= argv[12].asInt();
	mfd->fEnabled[2]	= argv[13].asInt();
	
	mfd->iOffset_x[3]	= argv[14].asInt();
	mfd->iOffset_y[3]	= argv[15].asInt();
	mfd->iFrame[3]		= argv[16].asInt();
	mfd->fEnabled[3]	= argv[17].asInt();

	mfd->iOffset_x[4]	= argv[18].asInt();
	mfd->iOffset_y[4]	= argv[19].asInt();
	mfd->iFrame[4]		= argv[20].asInt();
	mfd->fEnabled[4]	= argv[21].asInt();

	mfd->iOffset_x[5]	= argv[22].asInt();
	mfd->iOffset_y[5]	= argv[23].asInt();
	mfd->iFrame[5]		= argv[24].asInt();
	mfd->fEnabled[5]	= argv[25].asInt();

	mfd->iOffset_x[6]	= argv[26].asInt();
	mfd->iOffset_y[6]	= argv[27].asInt();
	mfd->iFrame[6]		= argv[28].asInt();
	mfd->fEnabled[6]	= argv[29].asInt();

	mfd->iOffset_x[7]	= argv[30].asInt();
	mfd->iOffset_y[7]	= argv[31].asInt();
	mfd->iFrame[7]		= argv[32].asInt();
	mfd->fEnabled[7]	= argv[33].asInt();
	
	mfd->iOffset_x[8]	= argv[34].asInt();
	mfd->iOffset_y[8]	= argv[35].asInt();
	mfd->iFrame[8]		= argv[36].asInt();
	mfd->fEnabled[8]	= argv[37].asInt();

	mfd->iOffset_x[9]	= argv[38].asInt();
	mfd->iOffset_y[9]	= argv[39].asInt();
	mfd->iFrame[9]		= argv[40].asInt();
	mfd->fEnabled[9]	= argv[41].asInt();

	mfd->red			= argv[42].asInt();
	mfd->green			= argv[43].asInt();
	mfd->blue			= argv[44].asInt();

	mfd->fDebug			= argv[45].asInt();
	mfd->fDontCrop		= argv[46].asInt();
}

static ScriptFunctionDef resize_func_defs[]={
	{ (ScriptFunctionPtr)resize_script_config, "Config", "0iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii" },   
	{ NULL },
};

static CScriptObject resize_obj={
	NULL, resize_func_defs
};

static bool resize_script_line(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	_snprintf(buf, buflen, "Config(%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d)",
		      mfd->new_x, mfd->new_y, 
			  mfd->iOffset_x[0],  mfd->iOffset_y[0], mfd->iFrame[0], mfd->fEnabled[0], 
			  mfd->iOffset_x[1],  mfd->iOffset_y[1], mfd->iFrame[1], mfd->fEnabled[1], 
			  mfd->iOffset_x[2],  mfd->iOffset_y[2], mfd->iFrame[2], mfd->fEnabled[2], 
			  mfd->iOffset_x[3],  mfd->iOffset_y[3], mfd->iFrame[3], mfd->fEnabled[3], 
			  mfd->iOffset_x[4],  mfd->iOffset_y[4], mfd->iFrame[4], mfd->fEnabled[4], 
			  mfd->iOffset_x[5],  mfd->iOffset_y[5], mfd->iFrame[5], mfd->fEnabled[5], 
			  mfd->iOffset_x[6],  mfd->iOffset_y[6], mfd->iFrame[6], mfd->fEnabled[6], 
			  mfd->iOffset_x[7],  mfd->iOffset_y[7], mfd->iFrame[7], mfd->fEnabled[7], 
			  mfd->iOffset_x[8],  mfd->iOffset_y[8], mfd->iFrame[8], mfd->fEnabled[8], 
			  mfd->iOffset_x[9],  mfd->iOffset_y[9], mfd->iFrame[9], mfd->fEnabled[9], 
			  mfd->red, mfd->green, mfd->blue, mfd->fDebug,  mfd->fDontCrop);
	return true;
}

int resize_init(FilterActivation *fa, const FilterFunctions *ff);

FilterDefinition filterDef_resize={
	0,0,NULL,
	"Exotic Pan (1.1)",
	"Exotic Pan (based on Zoom 1.2 by Donald Graft and internal resize filter by Avery Lee)."
#ifdef USE_ASM
			"\n[Assembly optimized] [FPU optimized] [MMX optimized]"
#endif
			,
	"NNever2000",
	NULL,
	sizeof(MyFilterData),
	resize_init,
	NULL,
	resize_run,
	resize_param,
	resize_config,
	resize_string,
	resize_start,
	resize_stop,

	&resize_obj,
	resize_script_line,
};

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_tutorial;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
	if (!(fd_tutorial = ff->addFilter(fm, &filterDef_resize, sizeof(FilterDefinition))))
		return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

	return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
	ff->removeFilter(fd_tutorial);
}

int resize_init(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	INITIALIZE_VTBLS;
	mfd->new_x = 320;
	mfd->new_y = 240;

	mfd->iOffset_x[0] = 0;
	mfd->iOffset_y[0] = 0;
	mfd->iFrame[0]	  = 0;
	mfd->fEnabled[0]  = false;

	mfd->iOffset_x[1] = 0;
	mfd->iOffset_y[1] = 0;
	mfd->iFrame[1]	  = 0;
	mfd->fEnabled[1]  = false;

	mfd->iOffset_x[2] = 0;
	mfd->iOffset_y[2] = 0;
	mfd->iFrame[2]	  = 0;
	mfd->fEnabled[2]  = false;

	mfd->iOffset_x[3] = 0;
	mfd->iOffset_y[3] = 0;
	mfd->iFrame[3]	  = 0;
	mfd->fEnabled[3]  = false;

	mfd->iOffset_x[4] = 0;
	mfd->iOffset_y[4] = 0;
	mfd->iFrame[4]	  = 0;
	mfd->fEnabled[4]  = false;

	mfd->iOffset_x[5] = 0;
	mfd->iOffset_y[5] = 0;
	mfd->iFrame[5]	  = 0;
	mfd->fEnabled[5]  = false;

	mfd->iOffset_x[6] = 0;
	mfd->iOffset_y[6] = 0;
	mfd->iFrame[6]	  = 0;
	mfd->fEnabled[6]  = false;

	mfd->iOffset_x[7] = 0;
	mfd->iOffset_y[7] = 0;
	mfd->iFrame[7]	  = 0;
	mfd->fEnabled[7]  = false;

	mfd->iOffset_x[8] = 0;
	mfd->iOffset_y[8] = 0;
	mfd->iFrame[8]	  = 0;
	mfd->fEnabled[8]  = false;

	mfd->iOffset_x[9] = 0;
	mfd->iOffset_y[9] = 0;
	mfd->iFrame[9]	  = 0;
	mfd->fEnabled[9]  = false;

	mfd->iIntervals = 0;
	mfd->red = 255;
	mfd->green = 0;
	mfd->blue = 0;
	mfd->fDebug = false;
	mfd->fDontCrop = false;
	
	return 0;
}
